#ifndef _OBJ_H
#define _OBJ_H

/*
FILE TYPES:
	T3D:	[T]ext [3D] object file (raw text)
	B3D:	[B]inary [3D] object file (compiled bytes)

	Needs updated:
		Support [object] 'center' so to make verticies relative to the center and not globalized
		Find way to incorporate u,v coordinates in with the vertex and face lists of [objects]
		Find way to AUTOMATICALLY map texture to a face of any [object] [sphere] [cone] or [cylinder]
			Please send any helpful info to goku_supersaiyan@netzero.net
			
		Possible conflict with [sphere]
			Ok to use but might crash system
			BSP is not constructing one leaf for the sphere
				either due to my own tessilation algo or BSP code itself
*/

// *********************************************************
void read_OBJ(char *, Poly_List &);
void read_binary_OBJ(char *, Poly_List &);
// *********************************************************

// *********************************************************
void read_OBJ(char *file, Poly_List &polygons) {
	ifstream vin;
	ofstream bout;
	char buf[128];
	bool done=false;

	cout << "Inputing vertex file " << file << " ... ";
	vin.open(file, ios::in);
	if(!vin) {
		cout << "\n\tNo such file: " << file << endl;
		return;
	}
	strcpy(buf, file);
	strcpy(buf+strlen(buf)-3, "b3d");
	bout.open(buf, ios::out|ios::binary);

	memset(buf, 0, 128);
	skip_white(vin, buf);
	do {
		obj_count++;
		if(strcmp(buf, "[object]")==0)			
			if(!parse_object(vin, bout, polygons)) break;
		if(strcmp(buf, "[sphere]")==0)
			if(!parse_sphere(vin,bout,polygons)) break;
		if(strcmp(buf, "[cylinder]")==0)
			if(!parse_cylinder(vin,bout,polygons)) break;
		if(strcmp(buf, "[cone]")==0)
			if(!parse_cone(vin,bout,polygons)) break;

		do {
			vin.getline(buf, 128);
			if(vin.eof()) {
				done=true;
				break;
			}
		} while(buf[0]==0  ||  buf[0]=='#');
		buf[10]=0;
		if(strcmp(buf, "[end_file]")==0)
				done=true;
	} while(!done);
	vin.close();
	bout.close();
	cout << endl;
}

void read_binary_OBJ(char *file, Poly_List &polygons) {
	ifstream bin;
	char type;

	cout << "Inputing binary vertex file " << file << " ... ";
	bin.open(file, ios::in|ios::binary);
	if(!bin) {
		cout << "\n\tNo such file: " << file << endl;
		return;
	}

	do {
		bin.read((unsigned char*)&type, 1);
		if(bin.eof())
			break;
		switch (type) {
			case OBJECT:
				parse_binary_object(bin,polygons);
				break;
			case SPHERE:
				parse_binary_sphere(bin, polygons);
				break;
			case CYLINDER:
				parse_binary_cylinder(bin, polygons);
				break;
			case CONE:
				parse_binary_cone(bin, polygons);
				break;
			default: break;
		}
	} while(1);
	bin.close();
	cout << endl;
}

// *********************************************************
#endif